@@ -1541,13 +1541,8 @@ static void _free_raws(struct dm_list *raw_list)
 
 static void _text_destroy(struct format_type *fmt)
 {
-	/* FIXME out of place, but the main (cmd) pool has been already
-	 * destroyed and touching the fid (also via release_vg) will crash the
-	 * program */
-	dm_hash_destroy(fmt->orphan_vg->fid->metadata_areas_index);
-	dm_hash_destroy(fmt->orphan_vg->hostnames);
-	dm_pool_destroy(fmt->orphan_vg->fid->mem);
-	dm_pool_destroy(fmt->orphan_vg->vgmem);
+	if (fmt->orphan_vg)
+		free_orphan_vg(fmt->orphan_vg);
 
 	if (fmt->private) {
 		_free_dirs(&((struct mda_lists *) fmt->private)->dirs);
@@ -1613,7 +1608,8 @@ static int _text_pv_setup(const struct format_type *fmt,
 
 			/* Be sure it's not already in VG's format instance! */
 			if (!fid_get_mda_indexed(vg->fid, pvid, ID_LEN, mda_index)) {
-				pv_mda_copy = mda_copy(vg->fid->mem, pv_mda);
+				if (!(pv_mda_copy = mda_copy(vg->fid->mem, pv_mda)))
+					return_0;
 				fid_add_mda(vg->fid, pv_mda_copy, pvid, ID_LEN, mda_index);
 			}
 		}
@@ -2270,8 +2266,10 @@ struct format_type *create_text_format(struct cmd_context *cmd)
 	const struct dm_config_value *cv;
 	struct mda_lists *mda_lists;
 
-	if (!(fmt = dm_malloc(sizeof(*fmt))))
-		return_NULL;
+	if (!(fmt = dm_malloc(sizeof(*fmt)))) {
+		log_error("Failed to allocate text format type structure.");
+		return NULL;
+	}
 
 	fmt->cmd = cmd;
 	fmt->ops = &_text_handler;
@@ -2296,13 +2294,13 @@ struct format_type *create_text_format(struct cmd_context *cmd)
 
 	if (!(fmt->labeller = text_labeller_create(fmt))) {
 		log_error("Couldn't create text label handler.");
-		goto err;
+		goto bad;
 	}
 
 	if (!(label_register_handler(FMT_TEXT_NAME, fmt->labeller))) {
 		log_error("Couldn't register text label handler.");
 		fmt->labeller->ops->destroy(fmt->labeller);
-		goto err;
+		goto bad;
 	}
 
 	if ((cn = find_config_tree_node(cmd, "metadata/dirs"))) {
@@ -2310,13 +2308,13 @@ struct format_type *create_text_format(struct cmd_context *cmd)
 			if (cv->type != DM_CFG_STRING) {
 				log_error("Invalid string in config file: "
 					  "metadata/dirs");
-				goto err;
+				goto bad;
 			}
 
 			if (!_add_dir(cv->v.str, &mda_lists->dirs)) {
 				log_error("Failed to add %s to text format "
 					  "metadata directory list ", cv->v.str);
-				goto err;
+				goto bad;
 			}
 			cmd->independent_metadata_areas = 1;
 		}
@@ -2325,31 +2323,26 @@ struct format_type *create_text_format(struct cmd_context *cmd)
 	if ((cn = find_config_tree_node(cmd, "metadata/disk_areas"))) {
 		for (cn = cn->child; cn; cn = cn->sib) {
 			if (!_get_config_disk_area(cmd, cn, &mda_lists->raws))
-				goto err;
+				goto_bad;
 			cmd->independent_metadata_areas = 1;
 		}
 	}
 
-	if (!(fmt->orphan_vg = alloc_vg("text_orphan", cmd, fmt->orphan_vg_name))) {
-		dm_free(fmt);
-		return NULL;
-	}
+	if (!(fmt->orphan_vg = alloc_vg("text_orphan", cmd, fmt->orphan_vg_name)))
+		goto_bad;
 
 	fic.type = FMT_INSTANCE_AUX_MDAS;
 	fic.context.vg_ref.vg_name = fmt->orphan_vg_name;
 	fic.context.vg_ref.vg_id = NULL;
-	if (!(fid = _text_create_text_instance(fmt, &fic))) {
-		log_error("Failed to create format instance");
-		release_vg(fmt->orphan_vg);
-		goto err;
-	}
+	if (!(fid = _text_create_text_instance(fmt, &fic)))
+		goto_bad;
+
 	vg_set_fid(fmt->orphan_vg, fid);
 
 	log_very_verbose("Initialised format: %s", fmt->name);
 
 	return fmt;
-
-      err:
+bad:
 	_text_destroy(fmt);
 
 	return NULL;
