@@ -19,6 +19,7 @@
 #include "segtype.h"
 #include "lv_alloc.h"
 #include "archiver.h"
+#include "defaults.h"
 
 int attach_pool_metadata_lv(struct lv_segment *pool_seg, struct logical_volume *metadata_lv)
 {
@@ -215,6 +216,31 @@ int pool_has_message(const struct lv_segment *seg,
 	return 0;
 }
 
+int pool_below_threshold(const struct lv_segment *pool_seg)
+{
+	percent_t percent;
+	int threshold = PERCENT_1 *
+		find_config_tree_int(pool_seg->lv->vg->cmd,
+				     "activation/thin_pool_autoextend_threshold",
+				     DEFAULT_THIN_POOL_AUTOEXTEND_THRESHOLD);
+
+	/* Data */
+	if (!lv_thin_pool_percent(pool_seg->lv, 0, &percent))
+		return_0;
+
+	if (percent >= threshold)
+		return_0;
+
+	/* Metadata */
+	if (!lv_thin_pool_percent(pool_seg->lv, 1, &percent))
+		return_0;
+
+	if (percent >= threshold)
+		return_0;
+
+	return 1;
+}
+
 struct lv_segment *find_pool_seg(const struct lv_segment *seg)
 {
 	struct lv_segment *pool_seg;
