@@ -1528,11 +1528,18 @@ static int lvconvert_raid(struct logical_volume *lv, struct lvconvert_params *lp
 		_lvconvert_raid_repair_ask(cmd, &replace);
 
 		if (replace) {
-			if (!(failed_pvs = _failed_pv_list(lv->vg))) {
-				stack;
-				return ECMD_FAILED;
+			if (!(failed_pvs = _failed_pv_list(lv->vg)))
+				return_0;
+
+			if (!lv_raid_replace(lv, failed_pvs, lp->pvh)) {
+				log_error("Failed to replace faulty devices in"
+					  " %s/%s.", lv->vg->name, lv->name);
+				return 0;
 			}
-			return lv_raid_replace(lv, failed_pvs, lp->pvh);
+
+			log_print("Faulty devices in %s/%s successfully"
+				  " replaced.", lv->vg->name, lv->name);
+			return 1;
 		}
 
 		/* "warn" if policy not set to replace */
@@ -1729,7 +1736,8 @@ static int _lvconvert_single(struct cmd_context *cmd, struct logical_volume *lv,
 		return ECMD_FAILED;
 	}
 
-	if (arg_count(cmd, repair_ARG) && !(lv->status & MIRRORED)) {
+	if (arg_count(cmd, repair_ARG) &&
+	    !(lv->status & MIRRORED) && !(lv->status & RAID)) {
 		if (arg_count(cmd, use_policies_ARG))
 			return ECMD_PROCESSED; /* nothing to be done here */
 		log_error("Can't repair non-mirrored LV \"%s\".", lv->name);
